package ca.odell.glazedlists.example;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.GlazedLists;
import ca.odell.glazedlists.FunctionList;
import ca.odell.glazedlists.impl.swing.SwingThreadProxyEventList;
import ca.odell.glazedlists.jfreechart.EventListPieDataset;
import ca.odell.glazedlists.gui.TableFormat;
import ca.odell.glazedlists.swing.EventTableModel;

import javax.swing.*;
import java.awt.*;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Random;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.plot.PiePlot;
import org.jfree.data.general.PieDataset;

public class PieChartExample {

    public static class AmericanIdol {
        private String name;
        private int votes;

        public AmericanIdol(String name, int votes) {
            this.name = name;
            this.votes = votes;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public int getVotes() {
            return votes;
        }

        public void setVotes(int votes) {
            this.votes = votes;
        }

        public void incrementVotes() {
            this.votes++;
        }
    }

    public static void main(String[] args) throws InterruptedException {
        // create an EventList of AmericanIdol
        final EventList<AmericanIdol> idols = new BasicEventList<AmericanIdol>();
        idols.add(new AmericanIdol("Simon Cowell", 0));
        idols.add(new AmericanIdol("Paula Abdul", 0));
        idols.add(new AmericanIdol("Randy Jackson", 0));
        idols.add(new AmericanIdol("Ryan Seacrest", 0));

        // build a JTable
        String[] propertyNames = new String[] {"name", "votes"};
        String[] columnLabels = new String[] {"Name", "Votes"};
        TableFormat<AmericanIdol> tf = GlazedLists.tableFormat(AmericanIdol.class, propertyNames, columnLabels);
        JTable t = new JTable(new EventTableModel<AmericanIdol>(idols, tf));

        // place the table in a JFrame
        final JFrame f = new JFrame();
        f.setLayout(new BorderLayout());
        final JScrollPane scrollPane = new JScrollPane(t);
        scrollPane.setPreferredSize(new Dimension(500, 100));
        f.add(scrollPane, BorderLayout.CENTER);

        // build PieChart key and value function
        final FunctionList.Function<AmericanIdol, Comparable<String>> keyFunction = GlazedLists.beanFunction(AmericanIdol.class, "name");
        final FunctionList.Function<AmericanIdol, Number> valueFunction = GlazedLists.beanFunction(AmericanIdol.class, "votes");

        // build the PieChart
        final SwingThreadProxyEventList<AmericanIdol> swingEDTIdolsList = new SwingThreadProxyEventList<AmericanIdol>(idols);
        final PieDataset idolsPieDataset = new EventListPieDataset<AmericanIdol, String>(swingEDTIdolsList, keyFunction, valueFunction);
        final JFreeChart pieChart = new JFreeChart("American Idol Votes", new PiePlot(idolsPieDataset));

        // place the PieChart in the JFrame
        final ChartPanel pieChartPanel = new ChartPanel(pieChart, true);
        pieChartPanel.setPreferredSize(new Dimension(500, 500));
        f.add(pieChartPanel, BorderLayout.SOUTH);

        // show the frame
        f.pack();
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setLocationRelativeTo(null);
        f.setVisible(true);

        // vote one idol off each episode
        while (idols.size() > 1) {
            // clear the votes
            for (ListIterator<AmericanIdol> i = idols.listIterator(); i.hasNext();) {
                AmericanIdol idol = i.next();
                idol.setVotes(0);
                i.set(idol);
                Thread.sleep(300);
            }

            // simulate voting
            long quittingTime = System.currentTimeMillis() + 5000;
            Random dice = new Random();
            while (System.currentTimeMillis() < quittingTime) {
                int index = dice.nextInt(idols.size());
                AmericanIdol idol = idols.get(index);
                idol.incrementVotes();
                idols.set(index, idol);

                Thread.sleep(100);
            }

            // find the idol with the least votes
            AmericanIdol votedOff = idols.get(0);
            for (Iterator<AmericanIdol> i = idols.iterator(); i.hasNext();) {
                AmericanIdol idol = i.next();
                if (idol.getVotes() < votedOff.getVotes()) {
                    votedOff = idol;
                }
            }

            // vote off the idol with the least votes
            Thread.sleep(1000);
            idols.remove(votedOff);
            Thread.sleep(1000);
        }
    }
}